"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getHighlighter = getHighlighter;
exports.getSafeLanguage = getSafeLanguage;
exports.isValidLanguage = isValidLanguage;
var _core = require("@shikijs/core");
var _css = _interopRequireDefault(require("shiki/dist/langs/css.mjs"));
var _html = _interopRequireDefault(require("shiki/dist/langs/html.mjs"));
var _javascript = _interopRequireDefault(require("shiki/dist/langs/javascript.mjs"));
var _json = _interopRequireDefault(require("shiki/dist/langs/json.mjs"));
var _markdown = _interopRequireDefault(require("shiki/dist/langs/markdown.mjs"));
var _php = _interopRequireDefault(require("shiki/dist/langs/php.mjs"));
var _tsx = _interopRequireDefault(require("shiki/dist/langs/tsx.mjs"));
var _typescript = _interopRequireDefault(require("shiki/dist/langs/typescript.mjs"));
var _githubDark = _interopRequireDefault(require("shiki/dist/themes/github-dark.mjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable no-console */

const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
// Define each grammar explicitly to ensure proper loading
const languageMap = {
  tsx: _tsx.default,
  typescript: _typescript.default,
  javascript: _javascript.default,
  json: _json.default,
  html: _html.default,
  css: _css.default,
  markdown: _markdown.default,
  php: _php.default,
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{
        match: '.*',
        name: 'text.plain'
      }]
    }
  }
};

// Rest of the RegexEngine implementation remains the same...
class CustomRegexScanner {
  constructor(patterns) {
    this.regexes = patterns.map(pattern => {
      try {
        return new RegExp(pattern, 'g');
      } catch (e) {
        log('Error creating regex for pattern:', pattern, e);
        return /./g;
      }
    });
  }
  findNextMatchSync(string, startPosition) {
    const stringContent = string.toString();
    for (let i = 0; i < this.regexes.length; i++) {
      const regex = this.regexes[i];
      if (!regex) continue;
      regex.lastIndex = startPosition;
      const match = regex.exec(stringContent);
      if (match) {
        return {
          index: match.index,
          captureIndices: [{
            start: match.index,
            end: match.index + match[0].length,
            length: match[0].length
          }],
          scanner: this
        };
      }
    }
    return null;
  }
  dispose() {
    this.regexes = [];
  }
}
class CustomRegexString {
  constructor(content) {
    this.content = content;
  }
  toString() {
    return this.content;
  }
}
function createCustomRegexEngine() {
  return {
    createScanner(patterns) {
      return new CustomRegexScanner(patterns);
    },
    createString(content) {
      return new CustomRegexString(content);
    }
  };
}
let highlighterInstance = null;
async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance');
    const langs = highlighterInstance.getLoadedLanguages();
    log('Cached languages:', langs);
    return highlighterInstance;
  }
  try {
    log('Creating new highlighter instance...');

    // Load language definitions
    log('Loading language definitions...');
    const languageDefinitions = Object.values(languageMap);
    log('Language definitions loaded:', languageDefinitions.map(l => l.id));

    // Initialize custom regex engine
    log('Initializing regex engine...');
    const engine = createCustomRegexEngine();

    // Create the highlighter core with explicit language loading
    log('Creating highlighter core with:', {
      languageCount: languageDefinitions.length,
      engineCapabilities: Object.keys(engine),
      themeAvailable: Boolean(_githubDark.default)
    });
    highlighterInstance = await (0, _core.createHighlighterCore)({
      themes: [_githubDark.default],
      langs: languageDefinitions,
      engine
    });

    // Verify languages were loaded
    const loadedLangs = highlighterInstance.getLoadedLanguages();
    log('Successfully loaded languages:', loadedLangs);
    return highlighterInstance;
  } catch (error) {
    log('Error creating highlighter:', error);
    throw error;
  }
}
function isValidLanguage(lang) {
  return lang in languageMap;
}
function getSafeLanguage(lang) {
  return isValidLanguage(lang) ? lang : 'plaintext';
}
//# sourceMappingURL=SyntaxContext.js.map