/* eslint-disable no-console */
import type { HighlighterCore, PatternScanner, RegexEngine, RegexEngineString } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'

import cssGrammar from 'shiki/dist/langs/css.mjs'
import htmlGrammar from 'shiki/dist/langs/html.mjs'
import javascriptGrammar from 'shiki/dist/langs/javascript.mjs'
import jsonGrammar from 'shiki/dist/langs/json.mjs'
import markdownGrammar from 'shiki/dist/langs/markdown.mjs'
import phpGrammar from 'shiki/dist/langs/php.mjs'
import tsxGrammar from 'shiki/dist/langs/tsx.mjs'
import typescriptGrammar from 'shiki/dist/langs/typescript.mjs'
import githubDark from 'shiki/dist/themes/github-dark.mjs'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args)
  }
}

export type AvailableLanguage = 'php' | 'tsx' | 'typescript' | 'javascript' | 'json' | 'html' | 'css' | 'markdown' | 'plaintext'

// Define each grammar explicitly to ensure proper loading
const languageMap: Record<string, any> = {
  tsx: tsxGrammar,
  typescript: typescriptGrammar,
  javascript: javascriptGrammar,
  json: jsonGrammar,
  html: htmlGrammar,
  css: cssGrammar,
  markdown: markdownGrammar,
  php: phpGrammar,
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{ match: '.*', name: 'text.plain' }],
    },
  },
}

// Rest of the RegexEngine implementation remains the same...
class CustomRegexScanner implements PatternScanner {
  private regexes: RegExp[]

  constructor(patterns: string[]) {
    this.regexes = patterns.map((pattern) => {
      try {
        return new RegExp(pattern, 'g')
      }
      catch (e) {
        log('Error creating regex for pattern:', pattern, e)
        return /./g
      }
    })
  }

  findNextMatchSync(string: string | RegexEngineString, startPosition: number) {
    const stringContent = string.toString()

    for (let i = 0; i < this.regexes.length; i++) {
      const regex = this.regexes[i]
      if (!regex)
        continue

      regex.lastIndex = startPosition
      const match = regex.exec(stringContent)

      if (match) {
        return {
          index: match.index,
          captureIndices: [
            {
              start: match.index,
              end: match.index + match[0].length,
              length: match[0].length,
            },
          ],
          scanner: this,
        }
      }
    }
    return null
  }

  dispose(): void {
    this.regexes = []
  }
}

class CustomRegexString implements RegexEngineString {
  content: string

  constructor(content: string) {
    this.content = content
  }

  toString(): string {
    return this.content
  }
}

function createCustomRegexEngine(): RegexEngine {
  return {
    createScanner(patterns: string[]): PatternScanner {
      return new CustomRegexScanner(patterns)
    },

    createString(content: string): RegexEngineString {
      return new CustomRegexString(content)
    },
  }
}

let highlighterInstance: HighlighterCore | null = null

export async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance')
    const langs = highlighterInstance.getLoadedLanguages()
    log('Cached languages:', langs)
    return highlighterInstance
  }

  try {
    log('Creating new highlighter instance...')

    // Load language definitions
    log('Loading language definitions...')
    const languageDefinitions = Object.values(languageMap)
    log(
      'Language definitions loaded:',
      languageDefinitions.map(l => l.id),
    )

    // Initialize custom regex engine
    log('Initializing regex engine...')
    const engine = createCustomRegexEngine()

    // Create the highlighter core with explicit language loading
    log('Creating highlighter core with:', {
      languageCount: languageDefinitions.length,
      engineCapabilities: Object.keys(engine),
      themeAvailable: Boolean(githubDark),
    })

    highlighterInstance = await createHighlighterCore({
      themes: [githubDark],
      langs: languageDefinitions,
      engine,
    })

    // Verify languages were loaded
    const loadedLangs = highlighterInstance.getLoadedLanguages()
    log('Successfully loaded languages:', loadedLangs)

    return highlighterInstance
  }
  catch (error) {
    log('Error creating highlighter:', error)
    throw error
  }
}

export function isValidLanguage(lang: string): lang is AvailableLanguage {
  return lang in languageMap
}

export function getSafeLanguage(lang: string): AvailableLanguage {
  return isValidLanguage(lang) ? (lang as AvailableLanguage) : 'plaintext'
}
